package com.zanxiang.game.module.manage.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.module.manage.pojo.dto.KfWebSocketMsgDTO;
import com.zanxiang.game.module.manage.pojo.params.KfWebSocketMsgParam;
import com.zanxiang.game.module.manage.service.IKfRoomMsgService;
import com.zanxiang.game.module.mybatis.entity.KfRoomMsg;
import com.zanxiang.game.module.mybatis.mapper.KfRoomMsgMapper;
import com.zanxiang.module.util.bean.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import reactor.util.function.Tuple2;
import reactor.util.function.Tuples;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2024-02-23
 * @description : 客服聊天房间消息
 */
@Slf4j
@Service
public class KfRoomMsgServiceImpl extends ServiceImpl<KfRoomMsgMapper, KfRoomMsg> implements IKfRoomMsgService {

    @Override
    public Tuple2<KfWebSocketMsgDTO.PageBean, List<KfWebSocketMsgDTO.RoomMsgBean>> msgRoomHistory(Long roomId, KfWebSocketMsgParam.PageBean pageBean) {
        Page<KfRoomMsg> kfRoomMsgPage = super.page(new Page<>(pageBean.getPageNum(), pageBean.getPageSize()),
                new QueryWrapper<KfRoomMsg>().lambda()
                        .eq(KfRoomMsg::getRoomId, roomId)
                        .orderByDesc(KfRoomMsg::getCreateTime)
        );
        List<KfWebSocketMsgDTO.RoomMsgBean> roomMsgBeanList = kfRoomMsgPage.getRecords().stream()
                .map(this::transform).collect(Collectors.toList());
        return Tuples.of(this.transform(kfRoomMsgPage), roomMsgBeanList);
    }

    private KfWebSocketMsgDTO.RoomMsgBean transform(KfRoomMsg kfRoomMsg) {
        if (kfRoomMsg == null) {
            return null;
        }
        return BeanUtil.copy(kfRoomMsg, KfWebSocketMsgDTO.RoomMsgBean.class);
    }

    private KfWebSocketMsgDTO.PageBean transform(Page<KfRoomMsg> kfRoomMsgPage) {
        return KfWebSocketMsgDTO.PageBean.builder()
                .pageNum(kfRoomMsgPage.getCurrent())
                .pageSize(kfRoomMsgPage.getSize())
                .pageTotal(kfRoomMsgPage.getPages())
                .total(kfRoomMsgPage.getTotal())
                .build();
    }
}
